/*! @mainpage Space Filling Curve Layout Algorithm

@section Description

Layout a simple graph using the Space Filling Curve Layout Algorithm as described in:

[MUELDER, Chris; MA, Kwan-Liu. Rapid graph layout using space filling curves. IEEE Transactions on Visualization and Computer Graphics, 2008, 14. Jg., Nr. 6.](http://vis.cs.ucdavis.edu/papers/muelder_ma_rgl_2008.pdf "http://vis.cs.ucdavis.edu/papers/muelder_ma_rgl_2008.pdf")

@section Usage

The program takes as a command the filename of a Tulip graph file and displays the graph using the implemented layout algorithm. Without an argument the program displays a complete tree. Two batch files for the california and yeast data sets are provided:

[run-california.bat](../run-california.bat "../run-california.bat")

[run-yeast.bat](../run-yeast.bat "../run-yeast.bat")

The view can be scrolled using left click and drag or the cursor keys. Zooming is possible with the mouse wheel. The A and S keys decrease and increase the cluster spacing factor.

Network nodes are sized and colored according to their node degree. Edges have the same color as the incident node with more edges.

The yeast protein homology data set and a short description are available at:

http://vlado.fmf.uni-lj.si/pub/networks/data/bio/Yeast/Yeast.htm

[http://vlado.fmf.uni-lj.si/pub/networks/data/bio/Yeast/Yeast.htm](http://vlado.fmf.uni-lj.si/pub/networks/data/bio/Yeast/Yeast.htm "http://vlado.fmf.uni-lj.si/pub/networks/data/bio/Yeast/Yeast.htm")

The california data set of web pages matching the query "California" and links between them is available at:

http://www.cs.cornell.edu/courses/cs685/2002fa/

*/ 
 #include <tulip/TulipPluginHeaders.h>
 #include <tulip/ForEach.h>
 
 #include <string>
 //#include <algorithm> // for heap functions
 #include <cmath>
 #include <Eigen/Geometry> 
 
 using namespace std;
 using namespace tlp;
 
 using std::cout;
 using std::endl;
 using std::string;
 
 
 static const char * paramHelp[] = {
     // 
     "",
 
     // 
     ""
 };
 
 /*! max_heap_type stores the maximum change in modularity dQ_max for each row of the matrix dQ, and the associated communities i and j; 
 */
 struct max_heap_type
 {
	 /*! value of the heap element 
	 */
     float dQ_max;
	 /*! first community to which the value applies 
	 */
	 int i;
	 /*! second community to which the value applies
	 */
	 int j;
 
	 /*! construct instance with invalid values 
	 */
     max_heap_type()
     {
         i = j = -1;
         dQ_max = -std::numeric_limits<float>::infinity();
     }
 
	 /*! construct with preset values
	 */
	 max_heap_type(int _i, int _j, float val) : dQ_max(val), i(_i), j(_j)
     {
     }
 
	 /*! comparison operator for heap functions 
	 */
     bool operator <(const max_heap_type &b) const
     {
         return dQ_max < b.dQ_max;
     };
 };
 
 /*! public function for writing a heap element to a standard ostream
 */
 std::ostream& operator<<(std::ostream& stream, max_heap_type h)
 {
     stream << "(" << h.i << "," << h.j << "): " << h.dQ_max;
 
     return stream;
 }
 
 /*! sparse matrix implementation for fast modularity clustering algorithm, as described in:

[CLAUSET, Aaron; NEWMAN, Mark EJ; MOORE, Cristopher. Finding community structure in very large networks. Physical review E, 2004, 70. Jg., Nr. 6, S. 066111.](https://arxiv.org/pdf/cond-mat/0408187 "https://arxiv.org/pdf/cond-mat/0408187")

The matrix can store explicit zeros, and access to elements is only allowed if they have been set before.

The data structure allows insert, removal and updates of arbitrary elements and keeps track of the maximum element of each row and the whole matrix in an efficient manner.

Insert, removal and update of elements, including book-keeping required to keep track of maximum elements, can be performed in O(n log n). 
 */
 class MySparseMatrix
 {
	 /*! main index, organized in row major fashion 
	 */
     std::vector< std::map<int, float> > _row;
 
	 /*! main heap for largest elements of each row with non-zero elements 
	 */
     std::vector<max_heap_type> _H;
	 /*! indirect array of elements in main heap 
	 */
     std::vector<int> _H_ind; // indirect heap of _H
	 /*! vector of positions of heap elements in main heap 
	 */
     std::vector<int> _H_pos; // position of element of _H in heap
 
	 /*! max_heaps for rows with non-zero elements 
	 */
     std::vector< std::vector<max_heap_type> > _H_k;
	 /*! indirect arrays of elements in heap of each row 
	 */
     std::vector< std::vector< int > > _H_k_ind;
	 /*! vector of positions of elements in heap of each row
	 */
     std::vector< std::vector<int> > _H_k_pos; // position of element of _H in heap
	 /*! vector of maps between key value (column) and position in element array 
	 */
     std::vector< std::map<int, int> > _H_k_map; // index of element with key "column" in _H
 
	 /*! Print heap (for debugging).
	 @param H	the elements of the heap
	 @param H_ind	the indirect array of the heap
	 @param H_pos	the position vector of the heap
	 */
     void print_heap(std::vector<max_heap_type> & H, std::vector<int> & H_ind, std::vector<int> & H_pos)
     {
         cout << "H: ";
         for (int it = 0; it < H.size(); ++it)
         {
             cout << it << ": " << H[it] << ", ";
         }
         cout << endl;
 
         cout << "H_ind: ";
         for (int it = 0; it < H_ind.size(); ++it)
         {
             cout << it << ": " << H_ind[it] << ", ";
         }
         cout << endl;
 
         cout << "H_pos: ";
         for (int it = 0; it < H_pos.size(); ++it)
         {
             cout << it << ": " << H_pos[it] << ", ";
         }
         cout << endl;
     }
 
	 /*! Check if a given heap is empty.
    @param H_ind	the indirect array of the heap 
	 */
     bool heap_empty(std::vector<int> & H_ind)
     {
         return H_ind.size() <= 1;
     }
 
	 /*! Return size of heap.
	 @param H_ind	the indirect array of the heap
    @return the number of elements in the heap 
	 */
     int heap_size(std::vector<int> & H_ind)
     {
         return H_ind.size() - 1;
     }
 
	 /*! Perform an upheap operation after the heap has been changed.
    @param H	the elements of the heap
    @param H_ind	the indirect array of the heap
    @param H_pos	the position vector of the heap
    @param pos_heap	the heap position that has changed 
	*/
     void upheap(std::vector<max_heap_type> & H, std::vector<int> & H_ind, std::vector<int> & H_pos, int pos_heap)
     {
         assert(pos_heap >= 1 && pos_heap <= heap_size(H_ind) && "upheap: heap index out of range");
 
         int pos_vec = H_ind[pos_heap];
         max_heap_type h = H[pos_vec];
 
         while (pos_heap > 1 && H[H_ind[pos_heap / 2]] < h)
         {
             H_ind[pos_heap] = H_ind[pos_heap / 2];
 
             H_pos[H_ind[pos_heap]] = pos_heap;
 
             pos_heap /= 2;
         }
 
         H_ind[pos_heap] = pos_vec;
         H_pos[H_ind[pos_heap]] = pos_heap;
     }
 
	 /*! Perform a downheap operation after the heap has been changed.
	 @param H	the elements of the heap
	 @param H_ind	the indirect array of the heap
	 @param H_pos	the position vector of the heap
	 @param pos_heap	the heap position that has changed
	 */
     void downheap(std::vector<max_heap_type> & H, std::vector<int> & H_ind, std::vector<int> & H_pos, int pos_heap)
     {
         assert(pos_heap >= 1 && pos_heap <= heap_size(H_ind) && "downheap: heap index out of range");
 
         int pos_vec = H_ind[pos_heap];
         max_heap_type h = H[pos_vec];
 
         int N = heap_size(H_ind);
 
         bool do_loop = true;
         while (do_loop && pos_heap <= N / 2)
         {
             int pos_heap2 = 2 * pos_heap;
 
             if (pos_heap2 < N)
                 if (H[H_ind[pos_heap2]] < H[H_ind[pos_heap2 + 1]])
                     ++pos_heap2;
 
             if (h < H[H_ind[pos_heap2]])
             {
                 H_ind[pos_heap] = H_ind[pos_heap2];
                 H_pos[H_ind[pos_heap]] = pos_heap;
 
                 pos_heap = pos_heap2;
             }
             else
                 do_loop = false;
         }
 
         H_ind[pos_heap] = pos_vec;
         H_pos[H_ind[pos_heap]] = pos_heap;
     }
 
	 /*! Perform a heap update operation by changing the value of an existing heap element.
    @param H	the elements of the heap
    @param H_ind	the indirect array of the heap
    @param H_pos	the position vector of the heap
    @param pos_heap	the heap position that is updated
    @param h_new	the new value of the heap element 
	 */
     void heap_update(std::vector<max_heap_type> & H, std::vector<int> & H_ind, std::vector<int> & H_pos, int pos_heap, max_heap_type h_new)
     {
         assert(pos_heap >= 1 && pos_heap <= heap_size(H_ind) && "heap_update: heap index out of range");
 
         max_heap_type h = H[H_ind[pos_heap]];
 
         assert(h.i == h_new.i && "heap_update: invalid heap update");
 
         bool up = h < h_new;
 
         H[H_ind[pos_heap]] = h_new;
 
         if (up)
         {
             upheap(H, H_ind, H_pos, pos_heap);
         }
         else
             downheap(H, H_ind, H_pos, pos_heap);
 
     }
 
	 /*! Insert a new element into a heap.
    @param H	the elements of the heap
    @param H_ind	the indirect array of the heap
    @param H_pos	the position vector of the heap
    @param h	the value of the new heap element 
	*/
     void heap_insert(std::vector<max_heap_type> & H, std::vector<int> & H_ind, std::vector<int> & H_pos, max_heap_type h)
     {
         int pos_vec = H.size();
         int pos_heap = heap_size(H_ind) + 1;
 
         H.push_back(h);
         H_ind.push_back(pos_vec);
         H_pos.push_back(pos_heap);
 
         assert(H_pos[pos_vec] == pos_heap && H_ind[pos_heap] == pos_vec && "heap_insert error in update");
 
         upheap(H, H_ind, H_pos, pos_heap);
     }
 
	 /*! Perform a heap erase operation which removes an existing element from the heap.
    @param H	the elements of the heap
    @param H_ind	the indirect array of the heap
    @param H_pos	the position vector of the heap
    @param pos_heap	the heap position of the element to remove 
	 */
     void heap_erase(std::vector<max_heap_type> & H, std::vector<int> & H_ind, std::vector<int> & H_pos, int pos_heap)
     {
         assert(pos_heap >= 1 && pos_heap <= heap_size(H_ind) && "heap_erase: heap index out of range");
 
         int pos_vec = H_ind[pos_heap];
 
         int pos_heap2 = heap_size(H_ind);
         bool up = H[pos_vec] < H[H_ind[pos_heap2]];
 
         if (pos_heap != pos_heap2)
         {
             H_ind[pos_heap] = H_ind[pos_heap2];
             H_pos[H_ind[pos_heap]] = pos_heap;
             //H_ind[pos_heap2] = pos_vec;
         }
 
         H_pos[pos_vec] = -1;
 
         H_ind.pop_back();
 
         if (pos_heap != pos_heap2)
         {
             if (up)
                 upheap(H, H_ind, H_pos, pos_heap);
             else
                 downheap(H, H_ind, H_pos, pos_heap);
         }
 
         assert(H_pos[pos_vec] == -1 && "heap_update: error");
     }
 
 public:
	 /*! Construct sparse matrix with predefined size and number of elements in each row.
	 @param V	number of nodes
	 @param reserve	vector containing size of each row 
	 */
     MySparseMatrix(int V, std::vector<int> reserve) : _row(V), _H(V), _H_pos(V, -1), _H_k(V), _H_k_ind(V), _H_k_pos(V), _H_k_map(V)
     {
         _H_ind.reserve(V);
 
         _H_ind.push_back(-1);  // heap positions start at 1, position 0 is unused
 
         for (int it = 0; it < V; ++it)
         {
             _H[it].i = it;
         }
 
         for (int it = 0; it < reserve.size(); ++it)
         {
             int s = reserve[it];
 
             _H_k[it].reserve(s);
             _H_k_ind[it].reserve(s);
             _H_k_pos[it].reserve(s);
 
             _H_k_ind[it].push_back(-1);  // heap positions start at 1, position 0 is unused
 
         }
     }
 
	 /*! Print heap of maximum elements of each row. 
	 */
     void print_heap()
     {
         print_heap(_H, _H_ind, _H_pos);
     }
 
	 /*! Print heap of a given row.
    @param row	the heap to print 
	 */
     void print_heap(int row)
     {
         print_heap(_H_k[row], _H_k_ind[row], _H_k_pos[row]);
     }
 
	 /*! Check if max-heap of all rows is empty.
	@return true if the heap is empty, else false 
	 */
     bool heap_empty()
     {
         return heap_empty(_H_ind);
     }
 
	/*! Get the top element of all heaps. The heap must not be empty.
	@return  max_heap_type the maximum element of the matrix 
	*/
     max_heap_type heap_top()
     {
         if (heap_empty(_H_ind))
         {
             cout << "heap_top heap empty" << endl;
             throw "heap empty";
         }
 
         return _H[_H_ind[1]];
     }
 
     void update(int row, int col, float val)
     {
         assert(_row[row].count(col) > 0 && "update: error element not set");
 
         _row[row][col] = val;
 
         assert(_H_k_map[row].count(col) > 0 && "update: error _H_k_map element not set");
 
         int pos_vec = _H_k_map[row][col];
         max_heap_type h_new(row, col, val);
 
         heap_update(_H_k[row], _H_k_ind[row], _H_k_pos[row], _H_k_pos[row][pos_vec], h_new);
 
         bool up = _H[row] < _H_k[row][_H_k_ind[row][1]];
 
         _H[row] = _H_k[row][_H_k_ind[row][1]];
 
         if (up)
             upheap(_H, _H_ind, _H_pos, _H_pos[row]);
         else
             downheap(_H, _H_ind, _H_pos, _H_pos[row]);
     }
 
	 /*! Perform a matrix insert operation. The element to be inserted must not exist.
    @param row	the row index of the element
    @param col	the column index of the element
    @param val	the new value for the element 
	 */
     void insert(int row, int col, float val)
     {
         assert(_row[row].count(col) == 0 && "insert: error element already set");
 
         _row[row][col] = val;
 
         assert(_H_k_map[row].count(col) == 0 && "insert: error _H_k_map element already set");
 
         int pos_vec = _H_k[row].size();
 
         max_heap_type h(row, col, val);
 
         heap_insert(_H_k[row], _H_k_ind[row], _H_k_pos[row], h);
 
         _H_k_map[row][col] = pos_vec;
 
         if (/*row >= _H_pos.size() ||*/ _H_pos[row] == -1)
         {
             _H[row] = h;
 
             _H_ind.push_back(row);
             _H_pos[row] = heap_size(_H_ind);
 
             upheap(_H, _H_ind, _H_pos, _H_pos[row]);
         }
         else if (_H[row] < h)
         {
             _H[row] = h;
 
             upheap(_H, _H_ind, _H_pos, _H_pos[row]);
         }
     }
 
	 /*! Erase an element from the matrix. The element to be removed must exist.
    @param row	the row index of the element
    @param col	the column index of the element 
	 */
     void erase(int row, int col)
     {
         assert(_row[row].count(col) > 0 && "insert: error element not set");
 
         _row[row].erase(col);
 
         assert(_H_k_map[row].count(col) > 0 && "insert: error _H_k_map element not set");
 
         int pos_vec = _H_k_map[row][col];
         int pos_heap = _H_k_pos[row][pos_vec];
 
         heap_erase(_H_k[row], _H_k_ind[row], _H_k_pos[row], pos_heap);
 
         _H_k_map[row].erase(col);
 
         if (_H[row].j == col)
         {
             if (heap_empty(_H_k_ind[row]))
             {
                 heap_erase(_H, _H_ind, _H_pos, _H_pos[row]);
 
                 assert(_H_pos[row] == -1 && "erase: error");
                 //_H_pos[row] = -1;
             }
             else
             {
                 max_heap_type h = _H_k[row][_H_k_ind[row][1]];
 
                 heap_update(_H, _H_ind, _H_pos, _H_pos[row], h);
             }
         }
     }
 
	 /*! Retrieve the container for elements of row The container and its elements must not be modified this way.
    @param r	the row to retrieve
	@return the standard container for row r 
	 */
     std::map<int, float>& row(int r)
     {
         return _row[r];
     }
 
	 /*! Get the number of elements of a row.
	@param row	the selected row
	@return the number of elements in the selected row 
	 */
     int innerNonZero(int row)
     {
         return _row[row].size();
     }
 
	 /*! Remove all elements from a row and update heaps The row to be removed may already be empty.
	@param row	the row to remove 
	*/
     void setZero(int row)
     {
         _row[row].clear();
         _H_k[row].resize(0);
         _H_k_ind[row].resize(1);
         _H_k_ind[row][0] = -1;
         _H_k_pos[row].resize(0);
         _H_k_map[row].clear();
 
         //cout << "setZero, row: " << row << ", _H_pos[row]: " << _H_pos[row] << endl;
         //print_heap(_H, _H_ind, _H_pos);
 
         // row may already be empty if it only had 1 element before merge
         if (_H_pos[row] != -1)
         {
             //cout << "setZero row: " << row << ", _H_pos[row]: " << _H_pos[row] << endl;
             heap_erase(_H, _H_ind, _H_pos, _H_pos[row]);
         }
     }
 
	 /*! Print the whole matrix (for debugging).
    @param stream	print the matrix to this stream 
	 */
     void print(std::ostream& stream)
     {
         for (int k = 0; k < _row.size(); ++k)
         {
             if (_row[k].size() > 0)
             {
                 stream << "row " << k << ":";
 
                 for (std::map<int, float>::const_iterator it = _row[k].begin(); it != _row[k].end(); ++it)
                 {
                     stream << " (" << it->first << "): " << it->second;
                 }
 
                 stream << endl;
             }
         }
     }
 
 
 };
 
 /*! Public function for printing a matrix to a standard ostream (for debugging).
 */
 std::ostream& operator<<(std::ostream& stream, MySparseMatrix& matrix)
 {
     matrix.print(stream);
     return stream;
 }
 
 /*! Main class for the space filling curve layout plugin. 
 */
 class SFCLayoutAlgorithm :public LayoutAlgorithm {
 public:
 
     // This line is used to pass information about the current plug-in.
     PLUGININFORMATION("SFCLayout",
         "Stefan Fiedler",
         "2018/05/18",
         "layout a graph along a space filling curve",
         "0.1.0",
         "Layout")
 
         
         SFCLayoutAlgorithm(const PluginContext* context) :LayoutAlgorithm(context)
     {
         //  addOutParameter<ParameterType>("clustering","Help string","Default value");
         addInParameter<bool>("redo_clustering", "Help string", "0");
         addInParameter<float>("cluster_spacing", "Help string", "2");
         //addInParameter<StringCollection>("curve type","Help string","Hilbert;Gosper");
 
     }
 
     // Define the check method only if needed.
     // It can be used to check topological properties of the graph,
     // metric properties on graph elements or anything else you need.
     // bool check(std::string & errorMsg) {
     //   errorMsg="";
     //   return true;
     // }
 
	 /*! Typedef for matrix type.
	 */
     typedef MySparseMatrix matrix_type;

	 /*! The run method is the main method of the plugin.
    It will be called out if the pre-condition method (bool check (..)) returned true.
    It is the starting point of the algorithm.
    @return       true if the algorithm succeeded 
	 */
     bool run() {
         // initialize fast modularity clustering algorithm
 
         int V = graph->numberOfNodes(),
             E = graph->numberOfEdges();
 
		 Graph * subgraph, * subgraph2;

		 bool redo_clustering = true;
		 if (graph->getSubGraph("clustering"))
		 {
			 subgraph = graph->getSubGraph("clustering");
			 subgraph2 = graph->getSubGraph("original");
			 redo_clustering = false;
		 }
		 else
		 {
			 BooleanProperty *select = graph->getLocalProperty<BooleanProperty>("firstSelection");
			 select->setAllNodeValue(true);
			 select->setAllEdgeValue(false);
			 subgraph = graph->addSubGraph(select, "clustering");

			 select->setAllEdgeValue(true);
			 subgraph2 = graph->addSubGraph(select, "original");
		 }

         //LayoutProperty *viewLayout = graph->getLocalProperty<LayoutProperty>("viewLayout");
         LayoutProperty *viewLayout2 = subgraph2->getLocalProperty<LayoutProperty>("viewLayout");
 
         ColorProperty *viewColor = subgraph2->getLocalProperty<ColorProperty>("viewColor");
 
         SizeProperty *viewSize = subgraph2->getLocalProperty<SizeProperty>("viewSize");
 
		 if (redo_clustering)
		 {
			 std::vector<int> reserve(V);
			 {
				 node n1;
				 forEach(n1, graph->getNodes())
				 {
					 int i = n1.id;

					 reserve[i] = graph->deg(n1);
				 }
			 }

			 // dQ(i, j) is the matrix of changes in modularity if communities i and j are joined
			 matrix_type dQ(V, reserve);

			 // a[i] is the fraction of ends of edges attached to vertices in community i
			 std::vector<float> a(V);

			 // current root node of communities in subgraph
			 std::vector<node> node_map(V);

			 std::vector<int> community_size(V);

			 // current root node of clustering
			 node rootNode;

			 std::vector<node> connected_component;

			 node n1;
			 forEach(n1, graph->getNodes())
			 {
				 int i = n1.id;

				 node n2;
				 forEach(n2, graph->getInOutNodes(n1))
				 {
					 int j = n2.id;

					 // ignore self-edges
					 if (i == j)
					 {
						 continue;
					 }

					 // insert new elements in dQ
					 float dQ_ij = 1 / (2.0f * E) - graph->deg(n1) * graph->deg(n2) / (4.0f * E * E);
					 dQ.insert(i, j, dQ_ij);
				 }

				 a[i] = graph->deg(n1) / (2.0f * E);

				 node_map[i] = n1;

				 community_size[i] = 1;


			 }

			 //cout << "dQ: " << endl;
			 //cout << dQ << endl;

			 // merge communities until only one is left
			 for (int iteration = 0; iteration < V - 1; ++iteration)
			 {
				 //cout << endl << "it " << iteration + 1 << " / " << V - 1  << endl;

				 if (dQ.heap_empty())
					 break;

				 max_heap_type h = dQ.heap_top();

				 assert(h.j != -1 && "error community not empty");

				 // join communities i and j
				 int i = h.i, j = h.j;

				 //cout << ", join communities " << i << " + " << j << " dQ_max: " << h.dQ_max << "dQ.heap_top: " << dQ.heap_top() << endl;

				 //cout << "Matrix dQ" << endl;
				 //cout << dQ << endl;

				 assert(dQ.innerNonZero(i) > 0 && dQ.innerNonZero(j) > 0 && "error row empty");

				 // update j-th row of dQ
				 dQ.erase(i, j);
				 dQ.erase(j, i);

				 // add elements of i_th row to j_th row
				 std::map<int, float> & row_i = dQ.row(i);
				 std::map<int, float> row_j(dQ.row(j));
				 std::map<int, float>::const_iterator it_i = row_i.begin(), it_j = row_j.begin();

				 dQ.setZero(j);

				 bool has_i = it_i != row_i.end(),
					 has_j = it_j != row_j.end();

				 int num_elements = 0;
				 while (has_i || has_j)
				 {
					 ++num_elements;

					 int i_col = -1, j_col = -1;
					 if (has_i)
						 i_col = it_i->first;
					 if (has_j)
						 j_col = it_j->first;

					 int k;
					 float dQ_new;

					 if (has_i && ((has_j && (i_col < j_col)) || !(has_j)))
					 {
						 k = i_col;
						 dQ_new = it_i->second - 2 * a[j] * a[k];

						 ++it_i;
					 }
					 else if (has_j && ((has_i && (i_col > j_col)) || !(has_i)))
					 {
						 k = j_col;
						 dQ_new = it_j->second - 2 * a[i] * a[k];

						 ++it_j;
					 }
					 else if (has_i && has_j && (i_col == j_col))
					 {
						 k = j_col;
						 dQ_new = it_i->second + it_j->second;

						 ++it_i;
						 ++it_j;
					 }
					 else
					 {
						 assert(false && "merge error");
					 }

					 has_i = it_i != row_i.end();
					 has_j = it_j != row_j.end();

					 assert(k <= V && "error k out of bounds");

					 dQ.insert(j, k, dQ_new);

					 // update k-th row of dQ
					 if (k == i_col)
						 dQ.erase(k, i);

					 if (k == j_col)
						 dQ.update(k, j, dQ_new);
					 else
						 dQ.insert(k, j, dQ_new);
				 }

				 // remove ith row from dQ
				 dQ.setZero(i);

				 // update a_i
				 a[j] += a[i];
				 a[i] = 0;

				 // join communities in clustering graph
				 node n1 = node_map[i], n2 = node_map[j];

				 rootNode = subgraph->addNode();

				 node_map[j] = rootNode;

				 subgraph->addEdge(rootNode, n1);
				 subgraph->addEdge(rootNode, n2);

				 if (num_elements == 0)
				 {
					 // this community is a connected component of the graph with rootNode at top
					 connected_component.push_back(rootNode);
				 }

				 community_size[j] += community_size[i];
			 }

			 // connect all connected components in tree hierarchy for traversal
			 {
				 node n1 = connected_component.back();
				 connected_component.pop_back();

				 while (connected_component.size() > 0)
				 {
					 node n2 = connected_component.back();
					 connected_component.pop_back();

					 rootNode = subgraph->addNode();

					 subgraph->addEdge(rootNode, n1);
					 subgraph->addEdge(rootNode, n2);

					 n1 = rootNode;
				 }
			 }
		 }
 
         if (true)
         {
			 int max_deg = 0;

			 std::vector<node> nodeOrder;
             nodeOrder.reserve(V);
             std::vector<int> nodeDepth(subgraph->numberOfNodes());
             int maxDepth = 0;
 
             // traverse clustering tree to compute node order
			 node rootNode(subgraph->numberOfNodes() - 1);
 
             std::vector<node> traversalStack;
 
             traversalStack.push_back(rootNode);
 
             //cout << "visit node:" << endl;
 
             //int order = 0;
             nodeDepth[rootNode.id] = 0;
 
             while (traversalStack.size() > 0)
             {
                 node np = traversalStack.back();
                 traversalStack.pop_back();
 
                 int d = subgraph->outdeg(np);
 
 
                 if (d == 2)
                 {
                     std::vector<node> nc(2);
                     int c = 0;
                     node n;
                     forEach(n, subgraph->getOutNodes(np))
                     {
                         nc[c] = n;
                         ++c;
                     }
                     traversalStack.push_back(nc[1]);
                     traversalStack.push_back(nc[0]);
 
                     nodeDepth[nc[0].id] = nodeDepth[nc[1].id] = nodeDepth[np.id] + 1;
 
                     if (maxDepth < nodeDepth[np.id] + 1)
                         maxDepth = nodeDepth[np.id] + 1;
 
                     //cout << " " << np.id << " (" << d << ": " << nc[0] << "," << nc[1] << ")";
                 }
                 else if (d == 0)
                 {
                     nodeOrder.push_back(np);
 
					 if (max_deg < graph->deg(np))
						 max_deg = graph->deg(np);

					 //cout << " " << np.id << " (" << d << ")";
                 }
                 else
                 {
                     cout << endl << "error: node degree " << d << endl;
 
                     nodeOrder.push_back(np);
                 }
             }
             //cout << endl;
             /*
             cout << "node order :" << endl;
             for (int it = 0; it < nodeOrder.size(); ++it)
             {
                 cout << " " << nodeOrder[it].id;
             }
             cout << endl;
             */
 
             float k_spacing;
             //dataSet->get(string("cluster_spacing"), k_spacing);
			 graph->getAttribute("cluster_spacing", k_spacing);
			 cout << "cluster_spacing: " << k_spacing << endl;

             std::vector<float> nodeSpacing(V - 1);
             float sum_spacing = 0;
             for (int it = 0; it < nodeOrder.size() - 1; ++it)
             {
                 nodeSpacing[it] = std::pow(std::abs(nodeDepth[nodeOrder[it + 1].id] - nodeDepth[nodeOrder[it].id]) + 1, k_spacing);
                 sum_spacing += nodeSpacing[it];
             }
 
             /*
             cout << "node spacing:" << endl;
             for (int it = 0; it < nodeOrder.size() - 1; ++it)
             {
                 cout << " " << nodeSpacing[it];
             }
             cout << endl;
             */
 
             std::vector<float> position(V);
             position[0] = 0;
             for (int it = 1; it < nodeOrder.size(); ++it)
             {
                 position[it] = position[it - 1] + nodeSpacing[it - 1] / sum_spacing;
             }
 
             /*
             cout << "position:" << endl;
             for (int it = 0; it < nodeOrder.size(); ++it)
             {
                 cout << " " << position[it];
             }
             cout << endl;
             */
 
             Eigen::Matrix3f Ma, Mb, M;
             Ma << 0, 1, 0,
                 1, 0, 0,
                 0, 0, 1;
             Mb << 0, -1, 1,
                 -1, 0, 1,
                 0, 0, 1;
 
             for (int it = 0; it < nodeOrder.size(); ++it)
             {
                 M.setIdentity();
 
                 float p = position[it];
 
                 float x = 0, y = 0;
 
                 //cout << "position: " << position[it] << endl;
 
                 Eigen::Vector3f v;
                 v << 0, 0, 0;
 
                 int steps = 16;
 
                 float l = std::pow(2, steps);
 
                 while (steps > 0)
                 {
                     l *= 0.5f;
 
                     Eigen::Vector3f v2, v3;
                     v2 << 0, 0, 1;
                     float p_new = p;
                     if (p < 0.25f)
                     {
                         v2(0, 0) = 0;
                         v2(1, 0) = 0;
 
                     }
                     else if (p < 0.5f)
                     {
                         p_new -= 0.25f;
 
                         v2(0, 0) = 0;
                         v2(1, 0) = 1;
                     }
                     else if (p < 0.75f)
                     {
                         p_new -= 0.5f;
 
                         v2(0, 0) = 1;
                         v2(1, 0) = 1;
                     }
                     else
                     {
                         p_new -= 0.75f;
 
                         v2(0, 0) = 1;
                         v2(1, 0) = 0;
                     }
                     p_new *= 4;
 
                     v3 = M * v2 * l;
                     v3(2, 0) = 0;
 
                     //cout << "step: " << steps << ", l: " << l << "p: " << p << ", p_new:" << p_new << ", v: " << v << endl;
                     //cout << "v2: " << v2 << endl;
                     //cout << "v3: " << v3 << endl;
                     //cout << "M: " << M << endl;
 
                     if (p < 0.25f)
                     {
                         M *= Ma;
                     }
                     else if (p >= 0.75f)
                     {
                         M *= Mb;
                     }
 
                     v = v + v3;
                     p = p_new;
                     --steps;
                 }
 
                 //cout << "node: " << nodeOrder[it].id << " (" << v(0, 0) << "," << v(1, 0) << ")" << endl;
 
                 result->setNodeValue(nodeOrder[it], Coord(v(0, 0), v(1, 0), 0.));
                 viewLayout2->setNodeValue(nodeOrder[it], Coord(v(0, 0), v(1, 0), 0.));
 
                 float s = std::sqrt((subgraph2->deg(nodeOrder[it]) * 1.0f) / max_deg);
                 viewColor->setNodeValue(nodeOrder[it], Color(255.0 * s, 128 * s + 64, 64 * s + 32));
 
                 s = (std::log(subgraph2->deg(nodeOrder[it])) + 1) * 10;
                 viewSize->setNodeValue(nodeOrder[it], tlp::Size(s, s, s));
             }
 
             {
                 edge e;
                 forEach(e, subgraph2->getEdges())
                 {
                     std::pair<node, node> p = subgraph2->ends(e);
                     if (subgraph2->deg(p.first) > subgraph2->deg(p.second))
                         viewColor->setEdgeValue(e, viewColor->getNodeValue(p.first));
                     else
                         viewColor->setEdgeValue(e, viewColor->getNodeValue(p.second));
                 }
             }
         }
         return true;
     }
 };
 // This second line will be used to register your algorithm in tulip using the information given above.
 PLUGIN(SFCLayoutAlgorithm)